package k4unl.minecraft.k4lib.client;

import k4unl.minecraft.k4lib.lib.Vector3fMax;
import net.minecraft.client.renderer.Tessellator;
import net.minecraft.client.renderer.WorldRenderer;
import net.minecraft.client.renderer.texture.TextureAtlasSprite;
import net.minecraft.client.renderer.texture.TextureMap;
import net.minecraft.client.renderer.vertex.DefaultVertexFormats;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.EnumFacing;
import net.minecraftforge.fml.client.FMLClientHandler;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;
import org.lwjgl.opengl.GL11;

@SideOnly(Side.CLIENT)
public class RenderHelper {

    static         float         lightBottom     = 0.5F;
    static         float         lightTop        = 1.0F;
    static         float         lightEastWest   = 0.8F;
    static         float         lightNorthSouth = 0.6F;
    private static Tessellator   tess            = Tessellator.func_178181_a();
    private static WorldRenderer worldRenderer   = tess.func_178180_c();
    public static  float         pixel           = 1.0F / 16.0F;
    public static  float         renderPixel     = 1.0F / 32.0F;
    public static  float         bigRenderPixel  = 1.0F / 64.0F;

    public static void vertexWithTexture(float x, float y, float z, float tL, float tT) {

        GL11.glTexCoord2f(tL, tT);
        GL11.glVertex3f(x, y, z);
    }

    public static void tesselatedTexture(float x, float y, float z, float tL, float tT) {
        worldRenderer.func_181662_b(x, y, z).func_181673_a(tL, tT).func_181675_d();
    }

    public static void startDrawingQuads() {
        worldRenderer.func_181668_a(GL11.GL_QUADS, DefaultVertexFormats.field_181707_g);
    }

    public static void tesselatorDraw() {
        tess.func_78381_a();
    }

    public static void drawCube(Vector3fMax vector) {
        drawTexturedCube(vector);
    }

    public static void drawCube(Vector3fMax vector, boolean doColors) {
        if (doColors) {
            drawColoredCube(vector);
        } else {
            drawCube(vector);
        }
    }

    public static void drawColoredCube(Vector3fMax vector) {
        //Top side
        GL11.glColor3f(1.0F, 0.0F, 0.0F);
        GL11.glVertex3f(vector.getXMin(), vector.getYMax(), vector.getZMax());
        GL11.glVertex3f(vector.getXMax(), vector.getYMax(), vector.getZMax());
        GL11.glVertex3f(vector.getXMax(), vector.getYMax(), vector.getZMin());
        GL11.glVertex3f(vector.getXMin(), vector.getYMax(), vector.getZMin());

        //Bottom side
        GL11.glColor3f(1.0F, 1.0F, 0.0F);
        GL11.glVertex3f(vector.getXMax(), vector.getYMin(), vector.getZMax());
        GL11.glVertex3f(vector.getXMin(), vector.getYMin(), vector.getZMax());
        GL11.glVertex3f(vector.getXMin(), vector.getYMin(), vector.getZMin());
        GL11.glVertex3f(vector.getXMax(), vector.getYMin(), vector.getZMin());

        //Draw west side:
        GL11.glColor3f(0.0F, 1.0F, 0.0F);
        GL11.glVertex3f(vector.getXMin(), vector.getYMin(), vector.getZMax());
        GL11.glVertex3f(vector.getXMin(), vector.getYMax(), vector.getZMax());
        GL11.glVertex3f(vector.getXMin(), vector.getYMax(), vector.getZMin());
        GL11.glVertex3f(vector.getXMin(), vector.getYMin(), vector.getZMin());

        //Draw east side:
        GL11.glColor3f(0.0F, 1.0F, 1.0F);
        GL11.glVertex3f(vector.getXMax(), vector.getYMin(), vector.getZMin());
        GL11.glVertex3f(vector.getXMax(), vector.getYMax(), vector.getZMin());
        GL11.glVertex3f(vector.getXMax(), vector.getYMax(), vector.getZMax());
        GL11.glVertex3f(vector.getXMax(), vector.getYMin(), vector.getZMax());

        //Draw north side
        GL11.glColor3f(0.0F, 0.0F, 1.0F);
        GL11.glVertex3f(vector.getXMin(), vector.getYMin(), vector.getZMin());
        GL11.glVertex3f(vector.getXMin(), vector.getYMax(), vector.getZMin());
        GL11.glVertex3f(vector.getXMax(), vector.getYMax(), vector.getZMin());
        GL11.glVertex3f(vector.getXMax(), vector.getYMin(), vector.getZMin());

        //Draw south side
        GL11.glColor3f(0.0F, 0.0F, 0.0F);
        GL11.glVertex3f(vector.getXMin(), vector.getYMin(), vector.getZMax());
        GL11.glVertex3f(vector.getXMax(), vector.getYMin(), vector.getZMax());
        GL11.glVertex3f(vector.getXMax(), vector.getYMax(), vector.getZMax());
        GL11.glVertex3f(vector.getXMin(), vector.getYMax(), vector.getZMax());
    }

    public static void setARGBFromHex(int hexColor) {
        float a = (hexColor >> 24 & 255) / 255.0F;
        float r = (hexColor >> 16 & 255) / 255.0F;
        float g = (hexColor >> 8 & 255) / 255.0F;
        float b = (hexColor & 255) / 255.0F;

        GL11.glColor4f(r, g, b, a);
    }

    public static void setRGBFromHex(int hexColor) {
        float r = (hexColor >> 16 & 255) / 255.0F;
        float g = (hexColor >> 8 & 255) / 255.0F;
        float b = (hexColor & 255) / 255.0F;

        GL11.glColor3f(r, g, b);
    }

    public static void drawCubeWithoutColor(Vector3fMax vector) {
        //Top side
        GL11.glVertex3f(vector.getXMin(), vector.getYMax(), vector.getZMax());
        GL11.glVertex3f(vector.getXMax(), vector.getYMax(), vector.getZMax());
        GL11.glVertex3f(vector.getXMax(), vector.getYMax(), vector.getZMin());
        GL11.glVertex3f(vector.getXMin(), vector.getYMax(), vector.getZMin());

        //Bottom side
        GL11.glVertex3f(vector.getXMax(), vector.getYMin(), vector.getZMax());
        GL11.glVertex3f(vector.getXMin(), vector.getYMin(), vector.getZMax());
        GL11.glVertex3f(vector.getXMin(), vector.getYMin(), vector.getZMin());
        GL11.glVertex3f(vector.getXMax(), vector.getYMin(), vector.getZMin());

        //Draw west side:
        GL11.glVertex3f(vector.getXMin(), vector.getYMin(), vector.getZMax());
        GL11.glVertex3f(vector.getXMin(), vector.getYMax(), vector.getZMax());
        GL11.glVertex3f(vector.getXMin(), vector.getYMax(), vector.getZMin());
        GL11.glVertex3f(vector.getXMin(), vector.getYMin(), vector.getZMin());

        //Draw east side:
        GL11.glVertex3f(vector.getXMax(), vector.getYMin(), vector.getZMin());
        GL11.glVertex3f(vector.getXMax(), vector.getYMax(), vector.getZMin());
        GL11.glVertex3f(vector.getXMax(), vector.getYMax(), vector.getZMax());
        GL11.glVertex3f(vector.getXMax(), vector.getYMin(), vector.getZMax());

        //Draw north side
        GL11.glVertex3f(vector.getXMin(), vector.getYMin(), vector.getZMin());
        GL11.glVertex3f(vector.getXMin(), vector.getYMax(), vector.getZMin());
        GL11.glVertex3f(vector.getXMax(), vector.getYMax(), vector.getZMin());
        GL11.glVertex3f(vector.getXMax(), vector.getYMin(), vector.getZMin());

        //Draw south side
        GL11.glVertex3f(vector.getXMin(), vector.getYMin(), vector.getZMax());
        GL11.glVertex3f(vector.getXMax(), vector.getYMin(), vector.getZMax());
        GL11.glVertex3f(vector.getXMax(), vector.getYMax(), vector.getZMax());
        GL11.glVertex3f(vector.getXMin(), vector.getYMax(), vector.getZMax());
    }

    /**
     * Draws a cube with the size of vector. Every face has the same color This uses the Tessellator
     *
     * @param vector
     * @author Koen Beckers (K4Unl)
     */

    public static void drawTesselatedCube(Vector3fMax vector) {

        boolean wasTesselating = false;

        // Check if we were already tesselating
        try {
            startDrawingQuads();
        } catch (IllegalStateException e) {
            wasTesselating = true;
        }

        // Top side
        worldRenderer.func_181666_a(1.0F, 0.0F, 0.0F, 0.7F);
        worldRenderer.func_181663_c(0, 1, 0);
        worldRenderer.func_181662_b(vector.getXMin(), vector.getYMax(), vector.getZMax());
        worldRenderer.func_181662_b(vector.getXMax(), vector.getYMax(), vector.getZMax());
        worldRenderer.func_181662_b(vector.getXMax(), vector.getYMax(), vector.getZMin());
        worldRenderer.func_181662_b(vector.getXMin(), vector.getYMax(), vector.getZMin());

        // Bottom side
        worldRenderer.func_181663_c(0, -1, 0);
        worldRenderer.func_181666_a(1.0F, 1.0F, 0.0F, 0.7F);
        worldRenderer.func_181662_b(vector.getXMax(), vector.getYMin(), vector.getZMax());
        worldRenderer.func_181662_b(vector.getXMin(), vector.getYMin(), vector.getZMax());
        worldRenderer.func_181662_b(vector.getXMin(), vector.getYMin(), vector.getZMin());
        worldRenderer.func_181662_b(vector.getXMax(), vector.getYMin(), vector.getZMin());

        // Draw west side:
        worldRenderer.func_181663_c(-1, 0, 0);
        worldRenderer.func_181666_a(0.0F, 1.0F, 0.0F, 0.7F);
        worldRenderer.func_181662_b(vector.getXMin(), vector.getYMin(), vector.getZMax());
        worldRenderer.func_181662_b(vector.getXMin(), vector.getYMax(), vector.getZMax());
        worldRenderer.func_181662_b(vector.getXMin(), vector.getYMax(), vector.getZMin());
        worldRenderer.func_181662_b(vector.getXMin(), vector.getYMin(), vector.getZMin());

        // Draw east side:
        worldRenderer.func_181663_c(1, 0, 0);
        worldRenderer.func_181666_a(0.0F, 1.0F, 1.0F, 0.7F);
        worldRenderer.func_181662_b(vector.getXMax(), vector.getYMin(), vector.getZMin());
        worldRenderer.func_181662_b(vector.getXMax(), vector.getYMax(), vector.getZMin());
        worldRenderer.func_181662_b(vector.getXMax(), vector.getYMax(), vector.getZMax());
        worldRenderer.func_181662_b(vector.getXMax(), vector.getYMin(), vector.getZMax());

        // Draw north side
        worldRenderer.func_181663_c(0, 0, -1);
        worldRenderer.func_181666_a(0.0F, 0.0F, 1.0F, 0.7F);
        worldRenderer.func_181662_b(vector.getXMin(), vector.getYMin(), vector.getZMin());
        worldRenderer.func_181662_b(vector.getXMin(), vector.getYMax(), vector.getZMin());
        worldRenderer.func_181662_b(vector.getXMax(), vector.getYMax(), vector.getZMin());
        worldRenderer.func_181662_b(vector.getXMax(), vector.getYMin(), vector.getZMin());

        // Draw south side
        worldRenderer.func_181663_c(0, 0, 1);
        worldRenderer.func_181666_a(0.0F, 0.0F, 0.0F, 0.7F);
        worldRenderer.func_181662_b(vector.getXMin(), vector.getYMin(), vector.getZMax());
        worldRenderer.func_181662_b(vector.getXMax(), vector.getYMin(), vector.getZMax());
        worldRenderer.func_181662_b(vector.getXMax(), vector.getYMax(), vector.getZMax());
        worldRenderer.func_181662_b(vector.getXMin(), vector.getYMax(), vector.getZMax());

        if (!wasTesselating) {
            tess.func_78381_a();
        }
    }


    public static boolean beginTesselatingWithTexture() {

        boolean wasTessellating = false;
        try {
            startDrawingQuads();
        } catch (IllegalStateException e) {
            wasTessellating = true;
        }
        FMLClientHandler.instance().getClient().func_110434_K().func_110577_a(TextureMap.field_110575_b);

        return wasTessellating;
    }

    public static void drawTesselatedSideTopWithTexture(Vector3fMax vector, TextureAtlasSprite icon) {
        //Top side
        worldRenderer.func_181663_c(0, 1, 0);
        tesselatedTexture(vector.getXMin(), vector.getYMax(), vector.getZMax(), icon.func_94209_e(), icon.func_94210_h()); //BL
        tesselatedTexture(vector.getXMax(), vector.getYMax(), vector.getZMax(), icon.func_94212_f(), icon.func_94210_h()); //BR
        tesselatedTexture(vector.getXMax(), vector.getYMax(), vector.getZMin(), icon.func_94212_f(), icon.func_94206_g()); //TR
        tesselatedTexture(vector.getXMin(), vector.getYMax(), vector.getZMin(), icon.func_94209_e(), icon.func_94206_g()); //TL
    }

    public static void drawTesselatedSideBottomWithTexture(Vector3fMax vector, TextureAtlasSprite icon) {
        //Bottom side
        worldRenderer.func_181663_c(0, -1, 0);
        tesselatedTexture(vector.getXMax(), vector.getYMin(), vector.getZMax(), icon.func_94212_f(), icon.func_94210_h()); //BR
        tesselatedTexture(vector.getXMin(), vector.getYMin(), vector.getZMax(), icon.func_94209_e(), icon.func_94210_h()); //BL
        tesselatedTexture(vector.getXMin(), vector.getYMin(), vector.getZMin(), icon.func_94209_e(), icon.func_94206_g()); //TL
        tesselatedTexture(vector.getXMax(), vector.getYMin(), vector.getZMin(), icon.func_94212_f(), icon.func_94206_g()); //TR
    }

    public static void drawTesselatedSideWestWithTexture(Vector3fMax vector, TextureAtlasSprite icon) {
        //Draw west side:
        worldRenderer.func_181663_c(-1, 0, 0);
        tesselatedTexture(vector.getXMin(), vector.getYMin(), vector.getZMax(), icon.func_94212_f(), icon.func_94206_g()); //BR
        tesselatedTexture(vector.getXMin(), vector.getYMax(), vector.getZMax(), icon.func_94212_f(), icon.func_94210_h()); //TR
        tesselatedTexture(vector.getXMin(), vector.getYMax(), vector.getZMin(), icon.func_94209_e(), icon.func_94210_h()); //TL
        tesselatedTexture(vector.getXMin(), vector.getYMin(), vector.getZMin(), icon.func_94209_e(), icon.func_94206_g()); //BL
    }

    public static void drawTesselatedSideEastWithTexture(Vector3fMax vector, TextureAtlasSprite icon) {
        //Draw east side:
        worldRenderer.func_181663_c(1, 0, 0);
        tesselatedTexture(vector.getXMax(), vector.getYMin(), vector.getZMin(), icon.func_94212_f(), icon.func_94210_h()); //BL
        tesselatedTexture(vector.getXMax(), vector.getYMax(), vector.getZMin(), icon.func_94212_f(), icon.func_94206_g()); //TL
        tesselatedTexture(vector.getXMax(), vector.getYMax(), vector.getZMax(), icon.func_94209_e(), icon.func_94206_g()); //TR
        tesselatedTexture(vector.getXMax(), vector.getYMin(), vector.getZMax(), icon.func_94209_e(), icon.func_94210_h()); //BR
    }

    public static void drawTesselatedSideNorthWithTexture(Vector3fMax vector, TextureAtlasSprite icon) {
        //Draw north side
        worldRenderer.func_181663_c(0, 0, -1);
        tesselatedTexture(vector.getXMin(), vector.getYMin(), vector.getZMin(), icon.func_94212_f(), icon.func_94206_g()); //BL
        tesselatedTexture(vector.getXMin(), vector.getYMax(), vector.getZMin(), icon.func_94212_f(), icon.func_94210_h()); //TL
        tesselatedTexture(vector.getXMax(), vector.getYMax(), vector.getZMin(), icon.func_94209_e(), icon.func_94210_h()); //TR
        tesselatedTexture(vector.getXMax(), vector.getYMin(), vector.getZMin(), icon.func_94209_e(), icon.func_94206_g()); //BR
    }

    public static void drawTesselatedSideSouthWithTexture(Vector3fMax vector, TextureAtlasSprite icon) {
        //Draw south side
        worldRenderer.func_181663_c(0, 0, 1);
        tesselatedTexture(vector.getXMax(), vector.getYMin(), vector.getZMax(), icon.func_94212_f(), icon.func_94210_h()); //BL
        tesselatedTexture(vector.getXMax(), vector.getYMax(), vector.getZMax(), icon.func_94212_f(), icon.func_94206_g()); //TL
        tesselatedTexture(vector.getXMin(), vector.getYMax(), vector.getZMax(), icon.func_94209_e(), icon.func_94206_g()); //TR
        tesselatedTexture(vector.getXMin(), vector.getYMin(), vector.getZMax(), icon.func_94209_e(), icon.func_94210_h()); //BR
    }

    public static void stopTesselating(boolean wasTessellating) {
        if (!wasTessellating) {
            tess.func_78381_a();
        }
    }

    public static void drawTesselatedCubeWithTexture(Vector3fMax vector, TextureAtlasSprite icon) {
        boolean wasTessellating = beginTesselatingWithTexture();
        drawTesselatedSideTopWithTexture(vector, icon);
        drawTesselatedSideBottomWithTexture(vector, icon);
        drawTesselatedSideWestWithTexture(vector, icon);
        drawTesselatedSideEastWithTexture(vector, icon);
        drawTesselatedSideNorthWithTexture(vector, icon);
        drawTesselatedSideSouthWithTexture(vector, icon);

        stopTesselating(wasTessellating);

    }


    public static void drawTexturedCube(Vector3fMax vector) {
        //Top side:
        RenderHelper.vertexWithTexture(vector.getXMin(), vector.getYMax(), vector.getZMax(), 0.0F, 0.0F);
        RenderHelper.vertexWithTexture(vector.getXMax(), vector.getYMax(), vector.getZMax(), 0.5F, 0.0F);
        RenderHelper.vertexWithTexture(vector.getXMax(), vector.getYMax(), vector.getZMin(), 0.5F, 0.5F);
        RenderHelper.vertexWithTexture(vector.getXMin(), vector.getYMax(), vector.getZMin(), 0.0F, 0.5F);

        //Bottom side:
        RenderHelper.vertexWithTexture(vector.getXMax(), vector.getYMin(), vector.getZMax(), 0.0F, 0.0F);
        RenderHelper.vertexWithTexture(vector.getXMin(), vector.getYMin(), vector.getZMax(), 0.5F, 0.0F);
        RenderHelper.vertexWithTexture(vector.getXMin(), vector.getYMin(), vector.getZMin(), 0.5F, 0.5F);
        RenderHelper.vertexWithTexture(vector.getXMax(), vector.getYMin(), vector.getZMin(), 0.0F, 0.5F);

        //Draw west side:
        RenderHelper.vertexWithTexture(vector.getXMin(), vector.getYMin(), vector.getZMax(), 0.5F, 0.0F);
        RenderHelper.vertexWithTexture(vector.getXMin(), vector.getYMax(), vector.getZMax(), 0.5F, 0.5F);
        RenderHelper.vertexWithTexture(vector.getXMin(), vector.getYMax(), vector.getZMin(), 0.0F, 0.5F);
        RenderHelper.vertexWithTexture(vector.getXMin(), vector.getYMin(), vector.getZMin(), 0.0F, 0.0F);

        //Draw east side:
        RenderHelper.vertexWithTexture(vector.getXMax(), vector.getYMin(), vector.getZMin(), 0.5F, 0.0F);
        RenderHelper.vertexWithTexture(vector.getXMax(), vector.getYMax(), vector.getZMin(), 0.5F, 0.5F);
        RenderHelper.vertexWithTexture(vector.getXMax(), vector.getYMax(), vector.getZMax(), 0.0F, 0.5F);
        RenderHelper.vertexWithTexture(vector.getXMax(), vector.getYMin(), vector.getZMax(), 0.0F, 0.0F);

        //Draw north side
        RenderHelper.vertexWithTexture(vector.getXMin(), vector.getYMin(), vector.getZMin(), 0.5F, 0.0F);
        RenderHelper.vertexWithTexture(vector.getXMin(), vector.getYMax(), vector.getZMin(), 0.5F, 0.5F);
        RenderHelper.vertexWithTexture(vector.getXMax(), vector.getYMax(), vector.getZMin(), 0.0F, 0.5F);
        RenderHelper.vertexWithTexture(vector.getXMax(), vector.getYMin(), vector.getZMin(), 0.0F, 0.0F);

        //Draw south side
        RenderHelper.vertexWithTexture(vector.getXMin(), vector.getYMin(), vector.getZMax(), 0.0F, 0.0F);
        RenderHelper.vertexWithTexture(vector.getXMax(), vector.getYMin(), vector.getZMax(), 0.5F, 0.0F);
        RenderHelper.vertexWithTexture(vector.getXMax(), vector.getYMax(), vector.getZMax(), 0.5F, 0.5F);
        RenderHelper.vertexWithTexture(vector.getXMin(), vector.getYMax(), vector.getZMax(), 0.0F, 0.5F);
    }

    public static void drawTexturedCubeWithLight(Vector3fMax vector, TileEntity t) {
        //TODO: FIX ME
        drawTexturedCube(vector);
        /*float light = t.blockType.getBlockBrightness(t.getWorldObj(), t.xCoord, t.yCoord, t.zCoord);
		light = (light + ((0.8f - light) * 0.4f)) * 0.9F;
		//light = 1.0F;
		
		int l = t.blockType.colorMultiplier(t.getWorldObj(), t.xCoord, t.yCoord, t.zCoord);
        float f = (l >> 16 & 255) / 255.0F;
        float f1 = (l >> 8 & 255) / 255.0F;
        float f2 = (l & 255) / 255.0F;
		//Tessellator tessellator = Tessellator.instance;
		
		//tessellator.startDrawingQuads();
		//tessellator.setBrightness(brightness);

		GL11.glColor3f(lightTop * light * f, lightTop * light * f1, lightTop * light * f2);
		//Top side:
		
		RenderHelper.vertexWithTexture(vector.getXMin(), vector.getYMax(), vector.getZMax(), 0.0F, 0.0F); //BL
		RenderHelper.vertexWithTexture(vector.getXMax(), vector.getYMax(), vector.getZMax(), 0.5F, 0.0F); //BR
		RenderHelper.vertexWithTexture(vector.getXMax(), vector.getYMax(), vector.getZMin(), 0.5F, 0.5F); //TR
		RenderHelper.vertexWithTexture(vector.getXMin(), vector.getYMax(), vector.getZMin(), 0.0F, 0.5F); //TL
		
		GL11.glColor3f(lightBottom * light * f, lightBottom * light * f1, lightBottom * light * f2);
		//Bottom side:
		RenderHelper.vertexWithTexture(vector.getXMax(), vector.getYMin(), vector.getZMax(), 0.0F, 0.0F);
		RenderHelper.vertexWithTexture(vector.getXMin(), vector.getYMin(), vector.getZMax(), 0.5F, 0.0F);		
		RenderHelper.vertexWithTexture(vector.getXMin(), vector.getYMin(), vector.getZMin(), 0.5F, 0.5F);
		RenderHelper.vertexWithTexture(vector.getXMax(), vector.getYMin(), vector.getZMin(), 0.0F, 0.5F);

		//Draw west side:
		GL11.glColor3f(lightEastWest * light * f, lightEastWest * light * f1, lightEastWest * light * f2);
		RenderHelper.vertexWithTexture(vector.getXMin(), vector.getYMin(), vector.getZMax(), 0.5F, 0.0F);
		RenderHelper.vertexWithTexture(vector.getXMin(), vector.getYMax(), vector.getZMax(), 0.5F, 0.5F);
		RenderHelper.vertexWithTexture(vector.getXMin(), vector.getYMax(), vector.getZMin(), 0.0F, 0.5F);
		RenderHelper.vertexWithTexture(vector.getXMin(), vector.getYMin(), vector.getZMin(), 0.0F, 0.0F);
		
		//Draw east side:
		GL11.glColor3f(lightEastWest * light, lightEastWest * light, lightEastWest * light);
		RenderHelper.vertexWithTexture(vector.getXMax(), vector.getYMin(), vector.getZMin(), 0.5F, 0.0F);
		RenderHelper.vertexWithTexture(vector.getXMax(), vector.getYMax(), vector.getZMin(), 0.5F, 0.5F);
		RenderHelper.vertexWithTexture(vector.getXMax(), vector.getYMax(), vector.getZMax(), 0.0F, 0.5F);
		RenderHelper.vertexWithTexture(vector.getXMax(), vector.getYMin(), vector.getZMax(), 0.0F, 0.0F);
		
		//Draw north side
		GL11.glColor3f(lightNorthSouth * light, lightNorthSouth * light, lightNorthSouth * light);
		RenderHelper.vertexWithTexture(vector.getXMin(), vector.getYMin(), vector.getZMin(), 0.5F, 0.0F); 
		RenderHelper.vertexWithTexture(vector.getXMin(), vector.getYMax(), vector.getZMin(), 0.5F, 0.5F);
		RenderHelper.vertexWithTexture(vector.getXMax(), vector.getYMax(), vector.getZMin(), 0.0F, 0.5F);
		RenderHelper.vertexWithTexture(vector.getXMax(), vector.getYMin(), vector.getZMin(), 0.0F, 0.0F);

		//Draw south side
		GL11.glColor3f(lightNorthSouth * light, lightNorthSouth * light, lightNorthSouth * light);
		RenderHelper.vertexWithTexture(vector.getXMin(), vector.getYMin(), vector.getZMax(), 0.0F, 0.0F);
		RenderHelper.vertexWithTexture(vector.getXMax(), vector.getYMin(), vector.getZMax(), 0.5F, 0.0F);
		RenderHelper.vertexWithTexture(vector.getXMax(), vector.getYMax(), vector.getZMax(), 0.5F, 0.5F);
		RenderHelper.vertexWithTexture(vector.getXMin(), vector.getYMax(), vector.getZMax(), 0.0F, 0.5F);
		
		GL11.glColor3f(1.0F, 1.0F, 1.0F);
		*/
    }
	
	public static void draw2DCircle(float xCenter, float yCenter, float r){
		
		GL11.glDisable(GL11.GL_TEXTURE_2D);
		GL11.glBegin(GL11.GL_TRIANGLES);
		int angle = 0;
		int resolution = 10;
		float dToR = (float)(Math.PI / 180.0);
		
		double prevX = xCenter + (r*Math.cos((angle-resolution)*dToR));
		double prevY = yCenter + (r*Math.sin((angle-resolution)*dToR));
		while(angle < 360){
			double x = xCenter + (r*Math.cos(angle*dToR));
			double y = yCenter + (r*Math.sin(angle*dToR));
			
			GL11.glVertex2f(xCenter, yCenter);
			GL11.glVertex2d(x, y);
			GL11.glVertex2d(prevX, prevY);
			prevX = x;
			prevY = y;
			
			angle+= resolution;
		}
		GL11.glEnd();
		GL11.glEnable(GL11.GL_TEXTURE_2D);
	}
	
	public static void drawCylinder(float xCenter, float yCenter, float zCenter, float r, float length, float xTextureStart, float yTextureStart, float xTextureEnd, float yTextureEnd){
		int ltd = 0;
		float x;
		float z;
		float x2;
		float z2;
		float dToR = (float)(Math.PI / 180.0);
		int reso = 20;

		GL11.glBegin(GL11.GL_QUADS);
		for(ltd = 0; ltd < 360; ltd+=reso){
			x = (float) (xCenter + (Math.sin(ltd * dToR) * r));
			z = (float) (zCenter + (Math.cos(ltd * dToR) * r));
			
			x2 = (float) (xCenter + (Math.sin((ltd+reso) * dToR) * r));
			z2 = (float) (zCenter + (Math.cos((ltd+reso) * dToR) * r));
			
			//GL11.glColor3f(ltd/360.0F, 0.0F, 0F);
			
			GL11.glTexCoord2f(xTextureStart, yTextureStart);
			GL11.glVertex3f(x2, yCenter, z2);
			
			GL11.glTexCoord2f(xTextureEnd, yTextureStart);
			GL11.glVertex3f(x2, yCenter+length, z2);
			
			GL11.glTexCoord2f(xTextureEnd, yTextureEnd);
			GL11.glVertex3f(x, yCenter+length, z);
			
			GL11.glTexCoord2f(xTextureStart, yTextureEnd);
			GL11.glVertex3f(x, yCenter, z);
		}
		GL11.glEnd();
	}
	

	
	public static void renderSide(Vector3fMax vector, EnumFacing dir){
		//Top side
		if(dir == EnumFacing.UP){
			GL11.glVertex3f(vector.getXMin(), vector.getYMax(), vector.getZMax());
			GL11.glVertex3f(vector.getXMax(), vector.getYMax(), vector.getZMax());
			GL11.glVertex3f(vector.getXMax(), vector.getYMax(), vector.getZMin());
			GL11.glVertex3f(vector.getXMin(), vector.getYMax(), vector.getZMin());
		}
		
		//Bottom side
		if(dir == EnumFacing.DOWN){
			GL11.glVertex3f(vector.getXMax(), vector.getYMin(), vector.getZMax());
			GL11.glVertex3f(vector.getXMin(), vector.getYMin(), vector.getZMax());
			GL11.glVertex3f(vector.getXMin(), vector.getYMin(), vector.getZMin());
			GL11.glVertex3f(vector.getXMax(), vector.getYMin(), vector.getZMin());
		}
		
		//West side
		if(dir == EnumFacing.WEST){
			GL11.glVertex3f(vector.getXMin(), vector.getYMin(), vector.getZMax());
			GL11.glVertex3f(vector.getXMin(), vector.getYMax(), vector.getZMax());
			GL11.glVertex3f(vector.getXMin(), vector.getYMax(), vector.getZMin());
			GL11.glVertex3f(vector.getXMin(), vector.getYMin(), vector.getZMin());
		}
		
		//East side
		if(dir == EnumFacing.EAST){
			GL11.glVertex3f(vector.getXMax(), vector.getYMin(), vector.getZMin());
			GL11.glVertex3f(vector.getXMax(), vector.getYMax(), vector.getZMin());
			GL11.glVertex3f(vector.getXMax(), vector.getYMax(), vector.getZMax());
			GL11.glVertex3f(vector.getXMax(), vector.getYMin(), vector.getZMax());
		}
		
		//North side
		if(dir == EnumFacing.NORTH){
			GL11.glVertex3f(vector.getXMin(), vector.getYMin(), vector.getZMin());
			GL11.glVertex3f(vector.getXMin(), vector.getYMax(), vector.getZMin());
			GL11.glVertex3f(vector.getXMax(), vector.getYMax(), vector.getZMin());
			GL11.glVertex3f(vector.getXMax(), vector.getYMin(), vector.getZMin());
		}
		
		//South side
		if(dir == EnumFacing.SOUTH){
			GL11.glVertex3f(vector.getXMin(), vector.getYMin(), vector.getZMax());
			GL11.glVertex3f(vector.getXMax(), vector.getYMin(), vector.getZMax());
			GL11.glVertex3f(vector.getXMax(), vector.getYMax(), vector.getZMax());
			GL11.glVertex3f(vector.getXMin(), vector.getYMax(), vector.getZMax());
		}
	}
	public static void drawCubeWithLines(int size, boolean isActive, float rF, float gF, float bF){
		float minPP = RenderHelper.pixel * (size+1);
		float minNP = RenderHelper.pixel * size;
		float maxPP = RenderHelper.pixel * (16-(size+1));
		float maxNP = RenderHelper.pixel * (16-size);
		
		Vector3fMax vNS = new Vector3fMax(minPP, minPP, minNP, maxPP, maxPP, maxNP);
		Vector3fMax vEW = new Vector3fMax(minNP, minPP, minPP, maxNP, maxPP, maxPP);
		Vector3fMax vTB = new Vector3fMax(minPP, minNP, minPP, maxPP, maxNP, maxPP);
		GL11.glColor3f(rF, gF, bF);
		RenderHelper.renderSide(vNS, EnumFacing.NORTH);
		RenderHelper.renderSide(vNS, EnumFacing.SOUTH);
		
		RenderHelper.renderSide(vEW, EnumFacing.EAST);
		RenderHelper.renderSide(vEW, EnumFacing.WEST);
		
		RenderHelper.renderSide(vTB, EnumFacing.UP);
		RenderHelper.renderSide(vTB, EnumFacing.DOWN);
		
		if(!isActive){
			GL11.glColor3f(1.0F, 0.0F, 0.0F);
		}else{
			GL11.glColor3f(0.0F, 1.0F, 0.0F);
		}
		Vector3fMax vEWS = new Vector3fMax(minNP, minNP, minNP, maxNP, maxNP, minPP);
		Vector3fMax vEWN = new Vector3fMax(minNP, minNP, maxPP, maxNP, maxNP, maxNP);
		
		Vector3fMax vEWT = new Vector3fMax(minNP, maxPP, minPP, maxNP, maxNP, maxPP);
		Vector3fMax vEWB = new Vector3fMax(minNP, minNP, minPP, maxNP, minPP, maxPP);
		
		Vector3fMax vNSW = new Vector3fMax(minNP, minNP, minNP, minPP, maxNP, maxNP);
		Vector3fMax vNSE = new Vector3fMax(maxPP, minNP, minNP, maxNP, maxNP, maxNP);
		Vector3fMax vNST = new Vector3fMax(minPP, maxPP, minNP, maxPP, maxNP, maxNP);
		Vector3fMax vNSB = new Vector3fMax(minPP, minNP, minNP, maxPP, minPP, maxNP);
		
		
		Vector3fMax vTBW = new Vector3fMax(minNP, minNP, minNP, minPP, maxNP, maxNP);
		Vector3fMax vTBE = new Vector3fMax(maxPP, minNP, minNP, maxNP, maxNP, maxNP);
		Vector3fMax vTBN = new Vector3fMax(minPP, minNP, minNP, maxPP, maxNP, minPP);
		Vector3fMax vTBS = new Vector3fMax(minPP, minNP, maxPP, maxPP, maxNP, maxNP);
		
		RenderHelper.renderSide(vEWS, EnumFacing.EAST);
		RenderHelper.renderSide(vEWS, EnumFacing.WEST);
		RenderHelper.renderSide(vEWN, EnumFacing.EAST);
		RenderHelper.renderSide(vEWN, EnumFacing.WEST);
		RenderHelper.renderSide(vEWT, EnumFacing.EAST);
		RenderHelper.renderSide(vEWT, EnumFacing.WEST);
		RenderHelper.renderSide(vEWB, EnumFacing.EAST);
		RenderHelper.renderSide(vEWB, EnumFacing.WEST);
		
		
		RenderHelper.renderSide(vNSW, EnumFacing.NORTH);
		RenderHelper.renderSide(vNSW, EnumFacing.SOUTH);
		RenderHelper.renderSide(vNSE, EnumFacing.NORTH);
		RenderHelper.renderSide(vNSE, EnumFacing.SOUTH);
		
		RenderHelper.renderSide(vNST, EnumFacing.NORTH);
		RenderHelper.renderSide(vNST, EnumFacing.SOUTH);
		RenderHelper.renderSide(vNSB, EnumFacing.NORTH);
		RenderHelper.renderSide(vNSB, EnumFacing.SOUTH);
		
		RenderHelper.renderSide(vTBW, EnumFacing.UP);
		RenderHelper.renderSide(vTBW, EnumFacing.DOWN);
		RenderHelper.renderSide(vTBE, EnumFacing.UP);
		RenderHelper.renderSide(vTBE, EnumFacing.DOWN);
		RenderHelper.renderSide(vTBN, EnumFacing.UP);
		RenderHelper.renderSide(vTBN, EnumFacing.DOWN);
		RenderHelper.renderSide(vTBS, EnumFacing.UP);
		RenderHelper.renderSide(vTBS, EnumFacing.DOWN);
	}
	
	public static void drawWhiteCube(Vector3fMax vector){
		//Top side
		GL11.glVertex3f(vector.getXMin(), vector.getYMax(), vector.getZMax());
		GL11.glVertex3f(vector.getXMax(), vector.getYMax(), vector.getZMax());
		GL11.glVertex3f(vector.getXMax(), vector.getYMax(), vector.getZMin());
		GL11.glVertex3f(vector.getXMin(), vector.getYMax(), vector.getZMin());
		
		//Bottom side
		GL11.glVertex3f(vector.getXMax(), vector.getYMin(), vector.getZMax());
		GL11.glVertex3f(vector.getXMin(), vector.getYMin(), vector.getZMax());
		GL11.glVertex3f(vector.getXMin(), vector.getYMin(), vector.getZMin());
		GL11.glVertex3f(vector.getXMax(), vector.getYMin(), vector.getZMin());
		
		//West side
		GL11.glVertex3f(vector.getXMin(), vector.getYMin(), vector.getZMax());
		GL11.glVertex3f(vector.getXMin(), vector.getYMax(), vector.getZMax());
		GL11.glVertex3f(vector.getXMin(), vector.getYMax(), vector.getZMin());
		GL11.glVertex3f(vector.getXMin(), vector.getYMin(), vector.getZMin());
		
		//East side
		GL11.glVertex3f(vector.getXMax(), vector.getYMin(), vector.getZMin());
		GL11.glVertex3f(vector.getXMax(), vector.getYMax(), vector.getZMin());
		GL11.glVertex3f(vector.getXMax(), vector.getYMax(), vector.getZMax());
		GL11.glVertex3f(vector.getXMax(), vector.getYMin(), vector.getZMax());
		
		//North side
		GL11.glVertex3f(vector.getXMin(), vector.getYMin(), vector.getZMin());
		GL11.glVertex3f(vector.getXMin(), vector.getYMax(), vector.getZMin());
		GL11.glVertex3f(vector.getXMax(), vector.getYMax(), vector.getZMin());
		GL11.glVertex3f(vector.getXMax(), vector.getYMin(), vector.getZMin());
		
		//South side
		GL11.glVertex3f(vector.getXMin(), vector.getYMin(), vector.getZMax());
		GL11.glVertex3f(vector.getXMax(), vector.getYMin(), vector.getZMax());
		GL11.glVertex3f(vector.getXMax(), vector.getYMax(), vector.getZMax());
		GL11.glVertex3f(vector.getXMin(), vector.getYMax(), vector.getZMax());
	}

    public static void drawGL11SideTopWithTexture(Vector3fMax vector, TextureAtlasSprite icon) {
        //Top side
        vertexWithTexture(vector.getXMin(), vector.getYMax(), vector.getZMax(), icon.func_94209_e(), icon.func_94210_h()); //BL
        vertexWithTexture(vector.getXMax(), vector.getYMax(), vector.getZMax(), icon.func_94212_f(), icon.func_94210_h()); //BR
        vertexWithTexture(vector.getXMax(), vector.getYMax(), vector.getZMin(), icon.func_94212_f(), icon.func_94206_g()); //TR
        vertexWithTexture(vector.getXMin(), vector.getYMax(), vector.getZMin(), icon.func_94209_e(), icon.func_94206_g()); //TL
    }

    public static void drawGL11SideBottomWithTexture(Vector3fMax vector, TextureAtlasSprite icon) {
        //Bottom side
        vertexWithTexture(vector.getXMax(), vector.getYMin(), vector.getZMax(), icon.func_94212_f(), icon.func_94210_h()); //BR
        vertexWithTexture(vector.getXMin(), vector.getYMin(), vector.getZMax(), icon.func_94209_e(), icon.func_94210_h()); //BL
        vertexWithTexture(vector.getXMin(), vector.getYMin(), vector.getZMin(), icon.func_94209_e(), icon.func_94206_g()); //TL
        vertexWithTexture(vector.getXMax(), vector.getYMin(), vector.getZMin(), icon.func_94212_f(), icon.func_94206_g()); //TR
    }

    public static void drawGL11SideWestWithTexture(Vector3fMax vector, TextureAtlasSprite icon) {
        //Draw west side:
        vertexWithTexture(vector.getXMin(), vector.getYMin(), vector.getZMax(), icon.func_94212_f(), icon.func_94210_h()); //BR
        vertexWithTexture(vector.getXMin(), vector.getYMax(), vector.getZMax(), icon.func_94212_f(), icon.func_94206_g()); //TR
        vertexWithTexture(vector.getXMin(), vector.getYMax(), vector.getZMin(), icon.func_94209_e(), icon.func_94206_g()); //TL
        vertexWithTexture(vector.getXMin(), vector.getYMin(), vector.getZMin(), icon.func_94209_e(), icon.func_94210_h()); //BL
    }

    public static void drawGL11SideEastWithTexture(Vector3fMax vector, TextureAtlasSprite icon) {
        //Draw east side:
        vertexWithTexture(vector.getXMax(), vector.getYMin(), vector.getZMin(), icon.func_94212_f(), icon.func_94210_h()); //BL
        vertexWithTexture(vector.getXMax(), vector.getYMax(), vector.getZMin(), icon.func_94212_f(), icon.func_94206_g()); //TL
        vertexWithTexture(vector.getXMax(), vector.getYMax(), vector.getZMax(), icon.func_94209_e(), icon.func_94206_g()); //TR
        vertexWithTexture(vector.getXMax(), vector.getYMin(), vector.getZMax(), icon.func_94209_e(), icon.func_94210_h()); //BR
    }

    public static void drawGL11SideNorthWithTexture(Vector3fMax vector, TextureAtlasSprite icon) {
        //Draw north side
        vertexWithTexture(vector.getXMin(), vector.getYMin(), vector.getZMin(), icon.func_94212_f(), icon.func_94210_h()); //BL
        vertexWithTexture(vector.getXMin(), vector.getYMax(), vector.getZMin(), icon.func_94212_f(), icon.func_94206_g()); //TL
        vertexWithTexture(vector.getXMax(), vector.getYMax(), vector.getZMin(), icon.func_94209_e(), icon.func_94206_g()); //TR
        vertexWithTexture(vector.getXMax(), vector.getYMin(), vector.getZMin(), icon.func_94209_e(), icon.func_94210_h()); //BR
    }

    public static void drawGL11SideSouthWithTexture(Vector3fMax vector, TextureAtlasSprite icon) {
        //Draw south side
        vertexWithTexture(vector.getXMax(), vector.getYMin(), vector.getZMax(), icon.func_94212_f(), icon.func_94210_h()); //BL
        vertexWithTexture(vector.getXMax(), vector.getYMax(), vector.getZMax(), icon.func_94212_f(), icon.func_94206_g()); //TL
        vertexWithTexture(vector.getXMin(), vector.getYMax(), vector.getZMax(), icon.func_94209_e(), icon.func_94206_g()); //TR
        vertexWithTexture(vector.getXMin(), vector.getYMin(), vector.getZMax(), icon.func_94209_e(), icon.func_94210_h()); //BR
    }

}
